#include <iostream>
#include <DGtal/base/Common.h>
#include <DGtal/io/boards/Board2D.h>
#include <DGtal/helpers/StdDefs.h>

using namespace DGtal;
using namespace DGtal::Z2i;

// Direction vector
Point direction(int dir)
{
	switch((dir%8+8)%8)
	{
		case 0:  return Point(+1, 0); break;
		case 1:  return Point(+1,+1); break;
		case 2:  return Point( 0,+1); break;
		case 3:  return Point(-1,+1); break;
		case 4:  return Point(-1, 0); break;
		case 5:  return Point(-1,-1); break;
		case 6:  return Point( 0,-1); break;
		case 7:  return Point(+1,-1); break;
		default: assert(false);
	}
}

// Build a circle or given center and radius
void makeCircle(DigitalSet& figure, const Point& center, float radius)
{
	for (const auto& p : figure.domain())
		if ((p - center).norm() <= radius)
			figure.insert(p);
}

// Walk on the coutour to build an orderer set (vector) of points
template<int STEP>
void getContour(std::vector<Point>& contour, const DigitalSet& figure)
{
	static_assert(STEP==1 || STEP==2);
	// Initial direction
	int   dir = 2; // DO NOT CHANGE
	// Find an entry point
	Point pt  = *figure.begin();
	while (figure(pt + Point(1,0)))
		pt += Point(1,0);
	// Walk on the border
	while (std::find(contour.begin(), contour.end(), pt) == contour.end())
	{
		contour.push_back(pt);
		int delta;
		for (delta = -2; delta < 6; delta += STEP)
			if (figure(pt + direction(dir + delta)))
			{
				dir += delta;
				pt  += direction(dir);
				break;
			}
	}
}

int main()
{
	Domain     domain(Point(-10, -10), Point(10,10));

	DigitalSet figure(domain);
	makeCircle(figure, Point(0,0), 8.2);

	std::vector<Point> contour;
	getContour<1>(contour, figure);
	// getContour<2>(contour, figure);

	// ┌────────────────────────────────────────────────────────────────────────┐
	// │                  Your work on the contour goes here !                  │
	// └────────────────────────────────────────────────────────────────────────┘

	Board2D board;
	board << domain;
	board << figure;
	board << CustomStyle(Point().className(), new CustomColors(Color(255,0,0), Color(255,0,0)));
	for (const auto& pt : contour)
		board << pt;

	board.saveEPS("testSet.eps");

	return 0;
}
